@@ -34,7 +34,7 @@ def gen_stru(stru_files, stru_type, pp_path, orb_path, tpath = ".", copy_pp_orb=
     # Find the pseudopotential files and orbital files
     pp_paths = comm.collect_pp(pp_path) # the return file name is pp_path/pp
     orb_paths = comm.collect_pp(orb_path)
-    recommand_ecutwfc = None if not os.path.isfile(os.path.join(pp_path, "ecutwfc.json")) else json.load(open(os.path.join(pp_path, "ecutwfc.json"), "r"))
+    recommand_ecutwfc = None if not pp_path or not os.path.isfile(os.path.join(pp_path, "ecutwfc.json")) else json.load(open(os.path.join(pp_path, "ecutwfc.json"), "r"))
     jobs = {}
     for ipath in stru_paths:
         istru = os.path.join(ipath, "STRU")
@@ -304,6 +304,24 @@ class AbacusStru:
         else:
             return self._mass
 
+    def globalidx2labelidx(self, idx):
+        """Get the label and index of the atom in that label from the global index
+        """
+        for i in range(len(self._atom_number)):
+            pre_atom_num = sum(self._atom_number[:i])
+            if idx >= pre_atom_num:
+                return self._label[i], idx - pre_atom_num
+        raise ValueError(f"The idx {idx} is larger than the total atom numbers!")
+    
+    def labelidx2globalidx(self, label, idx):
+        # Get the global index of the atom from the label and index of that label
+        assert (label in self._label), f"{label} is not valid, should be one of {self._label}"
+        label_idx = self._label.index(label)
+        if label_idx == 0:
+            return idx
+        else:
+            return sum(self._atom_number[:label_idx]) + idx
+    
     def get_atommag(self,norm = False):
         # return the magmom of each atom
         # if non-colinear, then return a list of three float for that atom.
@@ -942,6 +960,9 @@ class AbacusStru:
         generate a supercell
         
         na,nb,nc: the number of the supercell in a,b,c direction
+        
+        return a new AbacusStru object of the supercell.
+        The atom label order is same as the original structure, and the atom order of each label is repeatedly added based on the original order.
         '''
         # be careful, the real coord should be coord * lattice_constant
         na, nb, nc = nabc
@@ -985,7 +1006,118 @@ class AbacusStru:
                 setattr(new_stru,attri,[j for i in kv[attri] for j in i])
         new_stru._atom_number = [len(i) for i in coords]
         new_stru._check()
-        return new_stru   
+        return new_stru 
+    
+    def delete_atom(self, 
+                    label:str, 
+                    idx: Union[int, List[int], None] = None):
+        """Delete the atom of the element in del_element.
+        
+        label: str, the atomic label to be deleted, e.g. "H"
+        idx: int or list of int, the index of the atom to be deleted, if None, then delete all the atom of the element
+        """
+        if label not in self._label:
+            print(f"ERROR: the label {label} is not in the structure")
+            print("label:",self._label)
+            raise ValueError(f"the label {label} is not in the structure")
+        if isinstance(idx,int):
+            idx = [idx]
+        elif idx is None:
+            idx = list(range(self._atom_number[self._label.index(label)]))
+        elif isinstance(idx,list):
+            pass
+        else:
+            print(f"ERROR: the type of idx {type(idx)} is not supported")
+            raise ValueError(f"the type of idx {type(idx)} is not supported")
+        
+        if max(idx) >= self._atom_number[self._label.index(label)] or min(idx) < 0:
+            print(f"ERROR: the index in idx is out of range")
+            print("idx:",idx)
+            print("atom number of",label,":",self._atom_number[self._label.index(label)])
+            raise ValueError(f"the index in idx is out of range")
+
+        new_stru = copy.deepcopy(self)
+        global_idx = [sum(new_stru._atom_number[:new_stru._label.index(label)]) + i for i in idx]
+        new_stru._coord = [i for j, i in enumerate(new_stru._coord) if j not in global_idx]
+        new_stru._atom_number[new_stru._label.index(label)] -= len(idx)
+        if new_stru._atom_number[new_stru._label.index(label)] == 0:
+            del_idx = new_stru._label.index(label)
+            new_stru._label.pop(del_idx)
+            new_stru._mass.pop(del_idx)
+            new_stru._magmom.pop(del_idx)
+            if new_stru._pp:
+                new_stru._pp.pop(del_idx)
+            if new_stru._orb:
+                new_stru._orb.pop(del_idx)
+            if new_stru._paw:
+                new_stru._paw.pop(del_idx)
+            new_stru._atom_number.pop(del_idx)
+        key_p = ["_move","_magmom_atom","_velocity","_angle1","_angle2","_constrain","_lambda"]
+        for attri in key_p:
+            if getattr(new_stru,attri) is not None:
+                setattr(new_stru,attri,[i for j, i in enumerate(getattr(new_stru,attri)) if j not in global_idx])
+        new_stru._check()
+        return new_stru
+    
+    def set_empty_atom(self, label:str, idx: Union[int, List[int], None] = None):
+        """Set the specified atom to be empty by adding 'empty' to the label. Such as: H -> H_empty.
+        
+        This is useful for BSSE calculation, where some atoms are set to be empty.
+        
+        Args:
+            label: str, the atomic label to be set to empty, e.g. "H"
+            idx: int or list of int, the index of the atom to be set to empty, if None, then set all the atom of the element to empty
+        """    
+        if label not in self._label:
+            print(f"ERROR: the label {label} is not in the structure")
+            print("label:",self._label)
+            raise ValueError(f"the label {label} is not in the structure")
+        
+        label_idx = self._label.index(label)
+            
+        if idx is None:
+            new_label = label + "_empty"
+            self._label[label_idx] = new_label
+            return
+        
+        if isinstance(idx,int):
+            idx = [idx]
+        
+        if len(idx) == self._atom_number[label_idx]:
+            new_label = label + "_empty"
+            self._label[label_idx] = new_label
+            return
+        
+        atom_idx = [sum(self._atom_number[:label_idx]) + i for i in idx]
+        new_label = label + "_empty"
+        
+        self._label.append(new_label)
+        self._mass.append(self._mass[label_idx])
+        self._magmom.append(self._magmom[label_idx])
+        if self._pp:
+            self._pp.append(self._pp[label_idx])
+        if self._orb:
+            self._orb.append(self._orb[label_idx])
+        if self._paw:
+            self._paw.append(self._paw[label_idx])
+        self._atom_number.append(len(idx))
+        self._atom_number[label_idx] -= len(idx)
+
+        key_p = ["_coord","_move","_magmom_atom","_velocity","_angle1","_angle2","_constrain","_lambda"]
+        for attri in key_p:
+            if getattr(self,attri) is not None:
+                old_value = getattr(self,attri)
+                new_value = []
+                for j in range(len(old_value)):
+                    if j not in atom_idx:
+                        new_value.append(old_value[j])
+                for j in atom_idx:
+                    new_value.append(old_value[j])
+                setattr(self,attri,new_value)
+        self._check()
+                
+            
+        
      
     def write(self,struf="STRU"):
         cc = ""
@@ -1065,7 +1197,8 @@ class AbacusStru:
             cc += "\nNUMERICAL_DESCRIPTOR\n"
             cc += self._dpks    
 
-        Path(struf).write_text(cc)  
+        Path(struf).parent.mkdir(parents=True, exist_ok=True)
+        Path(struf).write_text(cc) 
     
     def write2poscar(self,poscar="POSCAR"):
         '''
